#include <stdio.h>
#include <time.h>
#include <errno.h>

#include "types.h"
#include "const.h"

FILE *recording = NULL;

#define REC_SEPARATOR "\n"

/* msleep(): Sleep for the requested number of milliseconds. */
int msleep(long msec)
{
    struct timespec ts;
    int res;

    if (msec < 0)
    {
        errno = EINVAL;
        return -1;
    }

    ts.tv_sec = msec / 1000;
    ts.tv_nsec = (msec % 1000) * 1000000;

    do
    {
        res = nanosleep(&ts, &ts);
    } while (res && errno == EINTR);

    return res;
}

int init_recording(char *file_name)
{
    recording = fopen(file_name, "w");
    return recording != NULL;
}

void append_record(
    board_t board,
    int size)
{
    if (recording)
    {
        for (int i = 0; i < size; i++)
        {
            for (int j = 0; j < size; j++)
            {
                fprintf(recording, "%3d ", board[i][j]);
            }
            fprintf(recording, "\n");
        }
        fprintf(recording, REC_SEPARATOR);
    }
}

void terminate_recording()
{
    if (recording)
    {
        fclose(recording);
        recording = NULL;
    }
}

void cleanup(board_t board, int size)
{
    for (int i = 0; i < size; i++)
        for (int j = 0; j < size; j++)
            if (board[i][j] < 0)
                board[i][j] = 0;
}

int do_swap(board_t board,
            int size,
            int swap,
            int r,
            int c)
{
    append_record(board, size);
    int other_r = r, other_c = c;
    switch (swap)
    {
    case SWAP_UP:
        if (r == 0)
            return SWAP_NULL;
        other_r--;
        break;
    case SWAP_DOWN:
        if (r == size - 1)
            return SWAP_NULL;
        other_r++;
        break;
    case SWAP_LEFT:
        if (c == 0)
            return SWAP_NULL;
        other_c--;
        break;
    case SWAP_RIGHT:
        if (c == size - 1)
            return SWAP_NULL;
        other_c++;
        break;
    }

    int aux = board[r][c];
    board[r][c] = board[other_r][other_c];
    board[other_r][other_c] = aux;

    return swap;
}
